import etl
import spacy
from spacy.matcher import PhraseMatcher


#
# SpaCy Named Entitiy Recognizer (NER)
#

# Appends classified (Persons, Locations, Organizations...) entities (labels/names/words) to mapped facets/fields

class enhance_ner_dictionary(object):

	def process (self, parameters={}, data={} ):
	
		verbose = False
		if 'verbose' in parameters:
			if parameters['verbose']:	
				verbose = True

		# default classifier
		classifier = 'en'

		if 'spacy_ner_classifier_default' in parameters:
			classifier = parameters['spacy_ner_classifier_default']

		# set language specific classifier, if configured and document language detected
		if 'spacy_ner_classifiers' in parameters and 'language_s' in data:
			# is a language speciic cassifier there for the detected language?
			if data['language_s'] in parameters['spacy_ner_classifiers']:
				classifier = parameters['spacy_ner_classifiers'][data['language_s']]

		if verbose:
			print ("Using SpaCY language model: {}".format(classifier))

		analyse_fields = ['title','content','description','ocr_t','ocr_descew_t']

		text = ''
		for field in analyse_fields:
			if field in data:
				text = "{}{}\n".format(text, data[field])

		# classify/tag with class each word of the content


		nlp = spacy.load('en')
		matcher = PhraseMatcher(nlp.vocab)
		
		entities_configfilename = "/etc/opensemanticsearch/entities/entities.tsv"
		entities_file = open(entities_configfilename, encoding="UTF-8")

		for line in entities_file:
			row = line.split("\t")

			facet = row[0]
			uri = row[1]
			preferred_label = row[2]
			labels = row[2:]

			patterns = [nlp(label.lower()) for label in labels]

			string_id = facet + "\t" + preferred_label
			matcher.add(string_id, None, *patterns)

		doc = nlp(text.lower())
		
		matches = matcher(doc)

		for match_id, start, end in matches:

			facet = nlp.vocab.strings[match_id].split("\t")[0]
			preferred_label = nlp.vocab.strings[match_id].split("\t")[1]
			label = str(doc[start:end])

			etl.append(data, facet, preferred_label)
			etl.append(data, facet + '_match_ss', label)

		# mark the document, that it was analyzed by this plugin yet
		data['enhance_ner_dictionary_b'] = "true"
		
		return parameters, data
